/**
 * Integrate JS Beautifier into Webuilder
 *
 * JS Beautifier is a coding style formatter for CSS/HTML/JavaScript. <https://www.npmjs.com/package/js-beautify>
 * You can easily write your own configuration to make your CSS/HTML/JavaScript beautiful and consistent.
 * Requires nodejs. <https://nodejs.org/>
 *
 * @category  WeBuilder Plugin
 * @package   CSS/HTML/JavaScript Beautifier Integration
 * @author    Peter Klein <pmk@io.dk>
 * @copyright 2017
 * @license   http://www.freebsd.org/copyright/license.html  BSD License
 * @version   2.02
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *     54   function InitBeautify()
 *     79   function InitCSSBeautify()
 *     93   function InitHTMLBeautify()
 *    107   function InitJSBeautify()
 *    121   function ValidateDocCodeTypes()
 *    185   function ShowBusyPanel(type)
 *    228   function BusyPanelOnShow()
 *    238   function StartBeautifying()
 *    292   function FileGetContents(filename)
 *    312   function FilePutContents(filename, contents)
 *    328   function OnInstalled()
 *
 * TOTAL FUNCTIONS: 11
 * (This index is automatically created/updated by the WeBuilder plugin "DocBlock Comments")
 *
 */

/**
 * Global variables
 *
 */
var beautifedText,
	textToBeautify,
	outputText,
	PSForm,
	curText,
	curCmd,
	curDocType,
	curCodeType;

/**
 * 3-in-1 Beautifying
 *
 * @return void
 */
function InitBeautify() {

	if (Editor.SelText !="") {
		switch (Document.CurrentCodeType) {
			case ltCSS: InitCSSBeautify();
			case ltHTML: InitHTMLBeautify();
			case ltJScript: InitJSBeautify();
			default: Alert("Not a valid code type!");
		}
	}
	else {
		switch (Document.DocType) {
			case dtCSS, dtSass, dtLess: InitCSSBeautify();
			case dtHTML: InitHTMLBeautify();
			case dtJScript: InitJSBeautify();
			default: Alert("Not a valid document type!");
		}
	}
}

/**
 * Setup variables for CSS Beautifying
 *
 * @return void
 */
function InitCSSBeautify() {
	curText = "StyleSheet";
	curCmd = "cssbeautify.cmd";
	curCfg = "cssbeautify.json";
	curDocType = [dtCSS, dtSass, dtLess];
	curCodeType = [ltCSS];
	ValidateDocCodeTypes();
}

/**
 * Setup variables for HTML Beautifying
 *
 * @return void
 */
function InitHTMLBeautify() {
	curText = "HTML";
	curCmd = "htmlbeautify.cmd";
	curCfg = "htmlbeautify.json";
	curDocType = [dtHTML];
	curCodeType = [ltHTML];
	ValidateDocCodeTypes();
}

/**
 * Setup variables for JavaScript Beautifying
 *
 * @return void
 */
function InitJSBeautify() {
	curText = "JavaScript";
	curCmd = "jsbeautify.cmd";
	curCfg = "jsbeautify.json";
	curDocType = [dtJScript];
	curCodeType = [ltJScript];
	ValidateDocCodeTypes();
}

/**
 * Check if document or selection codetype matches the beautyfier selected
 *
 * @return void
 */
function ValidateDocCodeTypes() {

	var enc = Document.Encoding;

	// Is something selected?
	if (Editor.SelText !="") {
		if ( Document.CurrentCodeType in curCodeType ) {

			// Workaround for encoding problem
			Editor.BeginEditing;
			if ((Script.ReadSetting("Convert Unicode to Entities", "0") == "1") && (enc == 0)) {
				Actions.Execute("ActCharsToEntities");
			}

			textToBeautify = Editor.SelText;
			type = "Selection";
		}
		else {
			Alert("Error: Selection is not " + curText + "!");
			return;
		}
	}
	// If not we grab the entire document
	else {
		if (Document.DocType in curDocType) {

			// Workaround for encoding problem
			Editor.BeginEditing;
			if ((Script.ReadSetting("Convert Unicode to Entities", "0") == "1") && (enc == 0)) {

				// Save selection/scroll position
				var sel = Editor.Selection,
					topLine = Editor.TopLine;

				Actions.Execute("ActSelectAll");
				Actions.Execute("ActCharsToEntities");

				// Restore selection/scroll position
				Editor.Selection = sel;
				Editor.TopLine = topLine;
			}

			textToBeautify = Editor.Text;
			type = "Document";
		}
		else {
			Alert("Error: File is not " + curText + "!");
			return;
		}
	}

	// Show busy panel and run beautifier
	ShowBusyPanel(type);
}

/**
 * Display Busy panel.
 * Prevents user from doing other actions while content is being beautified
 *
 * @param  string   type "Document" or "Selection"
 *
 * @return void
 */

function ShowBusyPanel(type) {

	PSForm = new TForm(WeBuilder);
	PSForm.Height = 70;
	PSForm.Width = 320;
	PSForm.Position = poScreenCenter;
	PSForm.BorderStyle = bsNone;  // disable dialog resizing
	PSForm.BorderIcons = 0; // remove close, maximize & minimize buttons
	PSForm.FormStyle = fsStayOnTop;
	PSForm.Color = clBtnFace;
	PSForm.Font.Color = clWindowText;
	PSForm.Font.Name = "Tahoma";
	PSForm.Font.Height = -21;
	PSForm.PixelsPerInch = 120;
	PSForm.OnShow = "BusyPanelOnShow";

	Panel1 = new TPanel(WeBuilder);
	Panel1.Parent = PSForm;
	Panel1.Align = alClient;

	Image1 = new TImage(WeBuilder);
	Image1.Parent = Panel1;
	Image1.SetBounds(10, 12, 45, 45);
    Image1.Picture.LoadFromFile(Script.Path + "/rose.png");

	Label1 = new TLabel(WeBuilder);
	Label1.Parent = Panel1;
	Label1.Layout = tlCenter;
	Label1.Alignment = taCenter;
	Label1.Caption = "Beautifying " + type + chr(13) + "Please Wait...";
	Label1.SetBounds(65, 0, 255, 70);

	PSForm.ShowModal;

	delete PSForm;

}

/**
 * Callback function for when the Busy Panel is displayed
 *
 * @return void
 */
function BusyPanelOnShow() {
	// We need a time delay, otherwise the closing of the modal window will be ignored.
	Script.TimeOut(1, "StartBeautifying");
}

/**
 * Integrate jsbeautify into Webuilder
 *
 * @return void
 */
function StartBeautifying() {
	var fileName = Document.FileName,	// Filename of current document
		ext = ExtractFileExt(fileName);	// File extension of current document

	// if name of file in Editor is "script.js", tempoary work file(name) will be "script-wbbwork.js"
	var workFileName = RegexReplace(fileName, Replace(ext,".","\\.") + "$", "", false) + "-wbbwork" + ext;

	// Remove existing *-wbbwork file to avoid double requesters popping up
	if (FileExists(workFileName)) {
	 	ExecuteCommand(Script.Path + "deletefile.cmd \"" + workFileName + "\"", null);
	}

	// Save a copy of original document
	FilePutContents(workFileName, textToBeautify);

	if (FileExists(workFileName)) {
		// If file exists (saved correctly), then we run jsbeautify
		var cmd = Script.Path + curCmd + " -r --config \"" + Script.Path + curCfg + "\" -f \"" + workFileName + "\"";
		if (ExecuteCommand(cmd, outputText)) {
			// Grab the content of the processed workfile
			beautifedText = FileGetContents(workFileName);

			// Remove the work file as we no longer need it
		 	ExecuteCommand(Script.Path + "deletefile.cmd \"" + workFileName + "\"", null);

			// Replace editor content with beautified content
			//Editor.BeginEditing;	// Moved to ValidateDocCodeTypes, in order to capture ActCharsToEntities
			if ((Editor.SelText !="")) Editor.SelText = beautifedText;
			else Editor.Text = beautifedText;
			Editor.EndEditing;

			PSForm.ModalResult = mrCancel;	// Close modal window
		}
		else {
			PSForm.ModalResult = mrCancel;	// Close modal window
			Alert("An error has occured. Unable to run beautifier.");
		}
	}
	else {
			PSForm.ModalResult = mrCancel;	// Close modal window
			Alert("An error has occured. Was not able to save workfile.");
	}

	// Just for safety
   	PSForm.ModalResult = mrCancel;	// Close modal window
}

/**
 * Load external file and return the contents
 *
 * @param  string   filename the name of the file to load
 *
 * @return string  the contents of the file
 */
function FileGetContents(filename) {
    var stream = CreateOleObject("ADODB.Stream");
    stream.Open;
    stream.Position = 0;
    stream.CharSet = "utf-8";
    stream.Type = 2;
    stream.LoadFromFile(filename);
    var contents = stream.ReadText();
    stream.Close;
	return contents;
}

/**
 * Save contents to external file
 *
 * @param  string   filename the name of the file to save
 * @param  string   contents the contents of the file
 *
 * @return void
 */
function FilePutContents(filename, contents) {
    var stream = CreateOleObject("ADODB.Stream");
    stream.Open;
    stream.Position = 0;
    stream.CharSet = "utf-8";
    stream.Type = 2;
    stream.WriteText(contents);
    stream.SaveToFile(filename, 2);
    stream.Close;
}

/**
 * Show info when plugin is installed
 *
 * @return void
 */
function OnInstalled() {
	alert("BeautifyWB 2.02 by Peter Klein installed sucessfully!");
}

Script.ConnectSignal("installed", "OnInstalled");

var bmp = new TBitmap, act;
LoadFileToBitmap(Script.Path + "rose_icon.png", bmp);
if (Script.ReadSetting("3-in-1 Beautifier", "1") == "1") {
	act = Script.RegisterDocumentAction("", "Beautifier", "", "InitBeautify");
	Actions.SetIcon(act, bmp);
}
else {
	act = Script.RegisterDocumentAction("Beautifier", "CSS Beautifier", "", "InitCSSBeautify");
	Actions.SetIcon(act, bmp);
	act = Script.RegisterDocumentAction("Beautifier", "HTML Beautifier", "", "InitHTMLBeautify");
	Actions.SetIcon(act, bmp);
	act = Script.RegisterDocumentAction("Beautifier", "JS Beautifier", "", "InitJSBeautify");
	Actions.SetIcon(act, bmp);
}
delete bmp;